@@ -137,6 +137,10 @@ If no encoding was done, nil is returned."
     (condition-case ()
 	(uudecode-decode-region (point-min) (point-max))
       (error nil)))
+   ((eq encoding 'x-binhex)
+    (condition-case ()
+	(binhex-decode-region (point-min) (point-max))
+      (error nil)))
    ((functionp encoding)
     (condition-case ()
 	(funcall encoding (point-min) (point-max))
